# meanvarx.mod   OOR2-AN-x-x
# AMPL coding: Sven Leyffer, University of Dundee, April 2000.
#
# From an original GAMS model:
#
#  Title Financial Optimization: Risk Management (MEANVARX,SEQ=113)
#  Minimum and maximum trade constraints are added to the standard
#  mean-variance model. If it is not profitable to trade within these
#  ranges, no trade should take place. A turnover constraint is added
#  to improve stability of the solution to small changes in data. The
#  resulting model is a nonlinear mixed-integer problem.
#
#  Reference: Dahl H, Meeraus A, and Zenios S, Some Financial Optimization
#             Models: Risk Management, in S Zenios (ed.), Financial
#             Optimization, Cambridge University Press, New York, NY, 1993.

set securities;				# ... set of securities

param mu {securities}; 			# ... expected return of security
param q {securities,securities};	# ... covariance matrix
param old {securities}; 		# ... current holdings fraction of portfolio
param umin {securities}; 		# ... min increase of holdings fraction
param umax {securities}; 		# ... max increase of holdings fraction
param lmin {securities}; 		# ... min decrease of holdings fraction
param lmax {securities}; 		# ... max decrease of holdings fraction

param tau := 0.5 ;      		# ... bound on turnover of current holdings
param lambda := 0.3;   			# ... return vs variance component tradeoff

var x {securities} >= 0;      		# ... fraction of portfolio of current holdings
var xi {securities} >= 0;    		# ... fraction of portfolio increae
var xd {securities} >= 0;    		# ... fraction of portfolio decrease
var mvar {i in securities}   		# ... marginal variance
    = sum{j in securities} q[i,j]*x[j];
var y {securities} binary;     		# ... binary for increasing current holdings
var z {securities} binary;     		# ... binary for decreasing current holdings

minimize objx: sum{i in securities} x[i]*mvar[i]- lambda*sum{i in securities} mu[i]*x[i];

subject to

   # ... budget constraint
   budget: sum{i in securities} x[i] = 1;

   # ... final portfolio definition
   xdef{i in securities}:  x[i] = old[i] - xd[i] + xi[i];

   # ... bound of maximum / minimum lot increase of fraction of i
   maxinc{i in securities}:  xi[i] <= umax[i]*y[i];
   mininc{i in securities}:  xi[i] >= umin[i]*y[i];

   # ... bound of maximum / minimum lot decrease of fraction of i
   maxdec{i in securities}:  xd[i] <= lmax[i]*z[i];
   mindec{i in securities}:  xd[i] >= lmin[i]*z[i];

   # ... restrict use of binary variables
   binsum{i in securities}: y[i] + z[i] <= 1;

   # ... restrict maximum turnover of portfolio
   turnover: sum{i in securities} xi[i] <= tau ;




